# Copyright (c) 2020 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import unicode_literals

import logging

from storops.unity.resource import UnityResource, UnityResourceList

LOG = logging.getLogger(__name__)


class UnityAlert(UnityResource):
    def modify(self, is_acknowledged=None):
        req_body = self._cli.make_body(isAcknowledged=is_acknowledged)
        resp = self.action('modify', **req_body)
        resp.raise_if_err()
        return resp

    def delete(self):
        resp = self._cli.delete(self.resource_class,
                                self.get_id())
        resp.raise_if_err()
        return resp


class UnityAlertList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityAlert
